@@ -12,8 +12,10 @@ import os
 import re
 import uuid
 from collections.abc import Callable
+from concurrent.futures import Future
 from dataclasses import dataclass
 from dataclasses import field
+from functools import partial
 from pathlib import Path
 from urllib.parse import ParseResult
 from urllib.parse import urlparse
@@ -31,11 +33,9 @@ from gajim.common.const import MIME_TYPES
 from gajim.common.helpers import load_file_async
 from gajim.common.helpers import write_file_async
 from gajim.common.i18n import _
+from gajim.common.multiprocess.thumbnail import create_thumbnail
 from gajim.common.storage.archive import models as mod
-from gajim.common.types import GdkPixbufType
 from gajim.common.util.http import create_http_request
-from gajim.common.util.image import create_thumbnail
-from gajim.common.util.image import get_pixbuf_from_data
 from gajim.common.util.preview import aes_decrypt
 from gajim.common.util.preview import filename_from_uri
 from gajim.common.util.preview import get_image_paths
@@ -91,7 +91,7 @@ class Preview:
         self.account = widget.account
         self.orig_path = orig_path
         self.thumb_path = thumb_path
-        self.size = size
+        self.size = size * app.window.get_scale_factor()
 
         self.thumbnail: bytes | None = None
         self.mime_type: str = ''
@@ -145,7 +145,7 @@ class Preview:
         return self._filename
 
     @property
-    def request(self) -> HTTPRequest:
+    def request(self) -> HTTPRequest | None:
         return self._request
 
     @property
@@ -176,15 +176,7 @@ class Preview:
             return False
         return self.orig_path.exists()
 
-    def create_thumbnail(self, data: bytes) -> bool:
-        self.thumbnail = create_thumbnail(data, self.size, self.mime_type)
-        if self.thumbnail is None:
-            self.info_message = _('Creating thumbnail failed')
-            log.warning('Creating thumbnail failed for: %s', self.orig_path)
-            return False
-        return True
-
-    def update_widget(self, data: GdkPixbufType | None = None) -> None:
+    def update_widget(self, data: bytes | None = None) -> None:
         self._widget.update(self, data)
 
     def update_progress(self, progress: float, request: HTTPRequest) -> None:
@@ -194,14 +186,8 @@ class Preview:
 
 class PreviewManager:
     def __init__(self) -> None:
-        self._orig_dir = Path(configpaths.get('MY_DATA')) / 'downloads'
-        self._thumb_dir = Path(configpaths.get('MY_CACHE')) / 'downloads.thumb'
-
-        if GLib.mkdir_with_parents(str(self._orig_dir), 0o700) != 0:
-            log.error('Failed to create: %s', self._orig_dir)
-
-        if GLib.mkdir_with_parents(str(self._thumb_dir), 0o700) != 0:
-            log.error('Failed to create: %s', self._thumb_dir)
+        self._orig_dir = configpaths.get('DOWNLOADS')
+        self._thumb_dir = configpaths.get('DOWNLOADS_THUMB')
 
         self._previews: dict[str, Preview] = {}
 
@@ -364,6 +350,35 @@ class PreviewManager:
                             self._on_thumb_load_finished,
                             preview)
 
+    def create_thumbnail(self, preview: Preview, data: bytes) -> None:
+        future = app.process_pool.submit(
+            create_thumbnail,
+            data,
+            preview.size,
+            preview.mime_type,
+        )
+        future.add_done_callback(partial(self._write_thumbnail, preview))
+
+    def _write_thumbnail(self, preview: Preview, future: Future[bytes | None]) -> None:
+        try:
+            result = future.result()
+            if result is None:
+                raise ValueError("result is None")
+        except Exception as error:
+            preview.info_message = _('Creating thumbnail failed')
+            preview.update_widget()
+            log.warning('Creating thumbnail failed for: %s %s',
+                        preview.orig_path, error)
+            return
+
+        preview.thumbnail = result
+
+        assert preview.thumb_path is not None
+        write_file_async(preview.thumb_path,
+                         preview.thumbnail,
+                         self._on_thumb_write_finished,
+                         preview)
+
     def _process_web_uri(self,
                          uri: str,
                          widget: Any,
@@ -388,7 +403,7 @@ class PreviewManager:
 
     def _on_orig_load_finished(self,
                                data: bytes | None,
-                               error: Gio.AsyncResult,
+                               error: GLib.Error | None,
                                preview: Preview) -> None:
         if preview.thumb_path is None or preview.orig_path is None:
             return
@@ -399,18 +414,15 @@ class PreviewManager:
 
         preview.mime_type = guess_mime_type(preview.orig_path, data)
         preview.file_size = os.path.getsize(preview.orig_path)
+
         if preview.is_previewable:
-            if preview.create_thumbnail(data):
-                assert preview.thumbnail is not None
-                write_file_async(preview.thumb_path,
-                                 preview.thumbnail,
-                                 self._on_thumb_write_finished,
-                                 preview)
-        preview.update_widget()
+            self.create_thumbnail(preview, data)
+        else:
+            preview.update_widget()
 
     @staticmethod
     def _on_thumb_load_finished(data: bytes | None,
-                                error: Gio.AsyncResult,
+                                error: GLib.Error | None,
                                 preview: Preview) -> None:
 
         if preview.thumb_path is None or preview.orig_path is None:
@@ -421,22 +433,12 @@ class PreviewManager:
             return
 
         preview.thumbnail = data
-        preview.mime_type = guess_mime_type(preview.orig_path, data)
+        # Thumbnails are stored always as PNG, we don’t know the
+        # mime-type of the original picture
+        preview.mime_type = guess_mime_type(preview.orig_path)
         preview.file_size = os.path.getsize(preview.orig_path)
 
-        try:
-            pixbuf = get_pixbuf_from_data(preview.thumbnail)
-        except Exception as err:
-            log.error('Unable to load: %s, %s',
-                      preview.thumb_path.name,
-                      err)
-            return
-
-        if pixbuf is None:
-            log.error('Unable to load pixbuf')
-            return
-
-        preview.update_widget(data=pixbuf)
+        preview.update_widget(data=data)
 
     def download_content(self,
                          preview: Preview,
@@ -490,7 +492,9 @@ class PreviewManager:
                             force: bool
                             ) -> None:
 
-        uri = request.get_uri().to_string()
+        uri = request.get_uri()
+        assert uri is not None
+        uri = uri.to_string()
         preview = cast(Preview, request.get_user_data())
         preview.mime_type = content_type
         preview.file_size = content_length
@@ -556,6 +560,7 @@ class PreviewManager:
             if preview.orig_path is not None:
                 preview.mime_type = guess_mime_type(preview.orig_path, data)
 
+        assert preview.orig_path is not None
         write_file_async(preview.orig_path,
                          data,
                          self._on_orig_write_finished,
@@ -566,14 +571,7 @@ class PreviewManager:
             return
 
         if preview.is_previewable:
-            if preview.create_thumbnail(data):
-                assert preview.thumb_path is not None
-                write_file_async(preview.thumb_path,
-                                 preview.thumbnail,
-                                 self._on_thumb_write_finished,
-                                 preview)
-            else:
-                preview.update_widget()
+            self.create_thumbnail(preview, data)
 
     @staticmethod
     def _on_orig_write_finished(_result: bool,
@@ -612,20 +610,9 @@ class PreviewManager:
         if preview.thumbnail is None:
             return
 
-        try:
-            pixbuf = get_pixbuf_from_data(preview.thumbnail)
-        except Exception as err:
-            log.error('Unable to load: %s, %s',
-                      preview.thumb_path.name,
-                      err)
-            return
-
-        if pixbuf is None:
-            log.error('Unable to load pixbuf')
-            return
-
-        preview.update_widget(data=pixbuf)
+        preview.update_widget(data=preview.thumbnail)
 
     def cancel_download(self, preview: Preview) -> None:
+        assert preview.request is not None
         preview.request.cancel()
         preview.download_in_progress = False
